"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const codeeditor_1 = require("@jupyterlab/codeeditor");
const signaling_1 = require("@phosphor/signaling");
class Editor {
    constructor(initialValue, editorFactory) {
        this._execute = new signaling_1.Signal(this);
        this._valueChanged = new signaling_1.Signal(this);
        this._model = new codeeditor_1.CodeEditor.Model({ value: initialValue });
        this._widget = new EditorWidget(this._model, editorFactory);
        this._model.value.changed.connect(() => {
            this._valueChanged.emit(this.value);
        });
        this._model.mimeType = 'text/x-sql';
        this._widget.executeCurrent.connect(() => {
            this._execute.emit(this.value);
        });
    }
    get value() {
        return this._model.value.text;
    }
    get widget() {
        return this._widget;
    }
    get execute() {
        return this._execute;
    }
    get valueChanged() {
        return this._valueChanged;
    }
}
exports.Editor = Editor;
class EditorWidget extends codeeditor_1.CodeEditorWrapper {
    constructor(model, editorFactory) {
        super({
            model,
            factory: editorFactory.newInlineEditor
        });
        this._executeCurrent = new signaling_1.Signal(this);
        this.editor.addKeydownHandler((_, evt) => this._onKeydown(evt));
        this.addClass('p-Sql-Editor');
    }
    get executeCurrent() {
        return this._executeCurrent;
    }
    _onKeydown(event) {
        if ((event.shiftKey || event.ctrlKey) && event.key === 'Enter') {
            this._executeCurrent.emit(void 0);
            return true;
        }
        return false;
    }
}
exports.EditorWidget = EditorWidget;
