"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const uuid = require("uuid");
const application_1 = require("@jupyterlab/application");
const apputils_1 = require("@jupyterlab/apputils");
const codeeditor_1 = require("@jupyterlab/codeeditor");
const launcher_1 = require("@jupyterlab/launcher");
const widget_1 = require("./widget");
const tracker_1 = require("./tracker");
require("../style/index.css");
function activate(app, palette, launcher, editorServices, restorer) {
    const tracker = tracker_1.createTracker();
    const command = 'jupyterlab-sql:open';
    restorer.restore(tracker, {
        command: command,
        args: widget => ({
            name: widget.content.name,
            connectionString: widget.content.toolbarModel.connectionString,
            sqlStatement: widget.content.sqlStatementValue
        }),
        name: widget => widget.content.name
    });
    app.commands.addCommand(command, {
        label: ({ isPalette }) => (isPalette ? 'New SQL session' : 'SQL'),
        iconClass: 'p-Sql-DatabaseIcon',
        execute: ({ name, connectionString, sqlStatement }) => {
            const widgetName = (name || uuid.v4());
            const initialConnectionString = ((connectionString || 'postgres://localhost:5432/postgres'));
            const initialSqlStatement = (sqlStatement || '');
            const widget = new widget_1.JupyterLabSqlWidget(editorServices.factoryService, {
                name: widgetName,
                initialConnectionString,
                initialSqlStatement
            });
            const main = new apputils_1.MainAreaWidget({ content: widget });
            app.shell.addToMainArea(main);
            tracker.add(main);
        }
    });
    palette.addItem({ command, category: 'SQL', args: { isPalette: true } });
    if (launcher) {
        launcher.add({ command, category: 'Other' });
    }
}
/**
 * Initialization data for the jupyterlab-sql extension.
 */
const extension = {
    id: 'jupyterlab-sql',
    autoStart: true,
    requires: [apputils_1.ICommandPalette, launcher_1.ILauncher, codeeditor_1.IEditorServices, application_1.ILayoutRestorer],
    activate
};
exports.default = extension;
