"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const widgets_1 = require("@phosphor/widgets");
const responseModel_1 = require("./responseModel");
const responseTable_1 = require("./responseTable");
class Response {
    constructor() {
        this._widget = new ResponseWidget();
    }
    get widget() {
        return this._widget;
    }
    setResponse(response) {
        this._widget.setResponse(response);
    }
}
exports.Response = Response;
class ResponseWidget extends widgets_1.Widget {
    constructor() {
        super();
        this._table = null;
        this.layout = new widgets_1.SingletonLayout();
    }
    dispose() {
        if (this._table) {
            this._table.dispose();
        }
        super.dispose();
    }
    onResize(msg) {
        if (this._item) {
            this._fitCurrentWidget();
        }
    }
    setResponse(response) {
        this._disposeTable();
        let widget;
        responseModel_1.ResponseModel.match(response, (keys, rows) => {
            const table = responseTable_1.ResponseTable.fromKeysRows(keys, rows);
            this._table = table;
            widget = table.widget;
        }, () => {
            const message = 'Command executed successfully';
            widget = new TextResponseWidget(message);
        }, ({ message }) => {
            widget = new TextResponseWidget(message);
        });
        this._setCurrentWidget(widget);
    }
    _setCurrentWidget(widget) {
        this.layout.widget = widget;
        this._item = new widgets_1.LayoutItem(this.layout.widget);
        this._fitCurrentWidget();
    }
    _fitCurrentWidget() {
        this._item.update(0, 0, this.node.offsetWidth, this.node.offsetHeight);
    }
    _disposeTable() {
        if (this._table) {
            this._table.dispose();
        }
        this._table = null;
    }
}
exports.ResponseWidget = ResponseWidget;
class TextResponseWidget extends widgets_1.Widget {
    constructor(message) {
        super();
        const element = document.createElement('div');
        const pre = document.createElement('pre');
        pre.innerHTML = message;
        element.appendChild(pre);
        this.node.appendChild(element);
    }
}
