"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const signaling_1 = require("@phosphor/signaling");
const apputils_1 = require("@jupyterlab/apputils");
const React = require("react");
const classnames_1 = require("classnames");
const services_1 = require("./services");
function newToolbar(model) {
    const container = new ToolbarContainer();
    container.model = model;
    return container;
}
exports.newToolbar = newToolbar;
class ToolbarModel extends apputils_1.VDomModel {
    constructor(initialConnectionString) {
        super();
        this._isLoading = false;
        this._connectionStringChanged = new signaling_1.Signal(this);
        this._connectionString = initialConnectionString;
    }
    get connectionStringChanged() {
        return this._connectionStringChanged;
    }
    get connectionString() {
        return this._connectionString;
    }
    get isLoading() {
        return this._isLoading;
    }
    set connectionString(newString) {
        this._connectionString = newString;
        this.stateChanged.emit(void 0);
        this._connectionStringChanged.emit(newString);
    }
    set isLoading(newValue) {
        this._isLoading = newValue;
        this.stateChanged.emit(void 0);
    }
}
exports.ToolbarModel = ToolbarModel;
class ToolbarContainer extends apputils_1.VDomRenderer {
    onConnectionStringChanged(newString) {
        if (!this.model) {
            return;
        }
        this.model.connectionString = newString;
    }
    render() {
        if (!this.model) {
            return null;
        }
        else {
            const connectionString = this.model.connectionString;
            const isLoading = this.model.isLoading;
            return (React.createElement("div", { className: "p-Sql-Toolbar" },
                React.createElement(ConnectionInformation, { connectionString: connectionString, onConnectionStringChanged: newString => this.onConnectionStringChanged(newString) }),
                isLoading && React.createElement(LoadingIcon, null)));
        }
    }
}
class ConnectionInformation extends React.Component {
    constructor(props) {
        super(props);
    }
    saveEdit(newConnectionString) {
        this.props.onConnectionStringChanged(newConnectionString);
    }
    render() {
        const { connectionString } = this.props;
        return (React.createElement(ConnectionInformationEdit, { connectionString: connectionString, onFinishEdit: (newConnectionString) => this.saveEdit(newConnectionString) }));
    }
}
class ConnectionInformationEdit extends React.Component {
    constructor(props) {
        super(props);
        this.inputRef = React.createRef();
        this.state = {
            value: services_1.ConnectionUrl.sanitize(props.connectionString),
            focused: false
        };
    }
    onKeyDown(event) {
        if (event.key === 'Enter') {
            this.inputRef.current.blur();
        }
        else if (event.keyCode === 27) {
            // ESC key
            this.cancel();
        }
    }
    onChange(event) {
        this.setState({ value: event.target.value });
    }
    start() {
        this.setState({
            focused: true,
            value: this.props.connectionString
        });
    }
    finish() {
        this.props.onFinishEdit(this.state.value);
        this.setState({
            focused: false,
            value: services_1.ConnectionUrl.sanitize(this.state.value)
        });
    }
    cancel() {
        this.setState({ value: this.props.connectionString });
    }
    onInputFocus() {
        this.start();
    }
    onInputBlur() {
        this.finish();
    }
    componentDidMount() {
        this.inputRef.current.focus();
    }
    render() {
        const { value, focused } = this.state;
        const inputWrapperClass = classnames_1.default('p-Sql-ConnectionInformation-input-wrapper', { 'p-mod-focused': focused });
        return (React.createElement("div", { className: inputWrapperClass },
            React.createElement("input", { className: "p-Sql-ConnectionInformation-text p-Sql-ConnectionInformation-input", value: value, ref: this.inputRef, onChange: event => this.onChange(event), onKeyDown: event => this.onKeyDown(event), onBlur: () => this.onInputBlur(), onFocus: () => this.onInputFocus() })));
    }
}
class LoadingIcon extends React.Component {
    render() {
        return (React.createElement("div", { className: "p-Sql-LoadingIcon" },
            React.createElement("div", { className: "p-Sql-LoadingIcon-Spinner" })));
    }
}
