"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const uuid = require("uuid");
const widgets_1 = require("@phosphor/widgets");
const signaling_1 = require("@phosphor/signaling");
const toolbar_1 = require("./toolbar");
const response_1 = require("./response");
const editor_1 = require("./editor");
const api_1 = require("./api");
class JupyterLabSqlWidget extends widgets_1.BoxPanel {
    constructor(editorFactory, options) {
        super();
        this._connectionStringChanged = new signaling_1.Signal(this);
        this._sqlStatementChanged = new signaling_1.Signal(this);
        this.name = options.name;
        this.id = 'jupyterlab-sql';
        this.title.label = 'SQL';
        this.title.closable = true;
        this.addClass('p-Sql-MainContainer');
        const toolbarModel = new toolbar_1.ToolbarModel(options.initialConnectionString);
        this.toolbarModel = toolbarModel;
        const connectionWidget = toolbar_1.newToolbar(toolbarModel);
        this.toolbarModel.connectionStringChanged.connect((_, value) => {
            this._connectionStringChanged.emit(value);
        });
        this.editor = new editor_1.Editor(options.initialSqlStatement, editorFactory);
        this.response = new response_1.Response();
        this.editor.execute.connect((_, value) => {
            const connectionString = this.toolbarModel.connectionString;
            this.updateGrid(connectionString, value);
        });
        this.editor.valueChanged.connect((_, value) => {
            this._sqlStatementChanged.emit(value);
        });
        this.addWidget(connectionWidget);
        this.addWidget(this.editor.widget);
        this.addWidget(this.response.widget);
        widgets_1.BoxPanel.setSizeBasis(connectionWidget, 50);
        widgets_1.BoxPanel.setStretch(this.editor.widget, 1);
        widgets_1.BoxPanel.setStretch(this.response.widget, 3);
    }
    get connectionStringChanged() {
        return this._connectionStringChanged;
    }
    get sqlStatementChanged() {
        return this._sqlStatementChanged;
    }
    get sqlStatementValue() {
        return this.editor.value;
    }
    updateGrid(connectionString, sql) {
        return __awaiter(this, void 0, void 0, function* () {
            const thisRequestId = uuid.v4();
            this._lastRequestId = thisRequestId;
            this.toolbarModel.isLoading = true;
            const data = yield api_1.Api.getForQuery(connectionString, sql);
            if (this._lastRequestId === thisRequestId) {
                // Only update the response widget if the current
                // query is the last query that was dispatched.
                this.response.setResponse(data);
            }
            this.toolbarModel.isLoading = false;
        });
    }
    onActivateRequest(msg) {
        this.editor.widget.activate();
    }
}
exports.JupyterLabSqlWidget = JupyterLabSqlWidget;
